@@ -15,7 +15,8 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright (C) 2012 Lanedo GmbH
-# Copyright (C) 2012-2017 Aleksander Morgado <aleksander@aleksander.es>
+# Copyright (C) 2012-2022 Aleksander Morgado <aleksander@aleksander.es>
+# Copyright (c) 2022 Qualcomm Innovation Center, Inc.
 #
 
 import string
@@ -32,7 +33,9 @@ class Field:
     """
     Constructor
     """
-    def __init__(self, prefix, dictionary, common_objects_dictionary, container_type, static):
+    def __init__(self, service, prefix, dictionary, common_objects_dictionary, container_type, static):
+        # The current QMI service
+        self.service = service
         # The field prefix, usually the name of the Container,
         #  e.g. "Qmi Message Ctl Something Output"
         self.prefix = prefix
@@ -58,8 +61,12 @@ class Field:
         if self.since is None:
             raise ValueError('TLV ' + self.fullname + ' requires a "since" tag specifying the major version where it was introduced')
 
+        # TLVs can no longer be structs, they must be sequences instead
+        if dictionary['format'] == 'struct':
+            raise ValueError('TLV ' + self.fullname + ' cannot have type "struct": use "sequence" instead')
+
         # Create our variable object
-        self.variable = VariableFactory.create_variable(dictionary, self.fullname, self.container_type)
+        self.variable = VariableFactory.create_variable(self.service, dictionary, self.fullname, self.container_type)
 
         # Create the variable name within the Container
         self.variable_name = 'arg_' + utils.build_underscore_name(self.name).lower()
@@ -85,6 +92,11 @@ class Field:
                     else:
                         raise RuntimeError('Common type \'%s\' not found' % prerequisite_dictionary['name'])
 
+        if 'personal-info' in dictionary:
+            self.personal_info = True;
+        else:
+            self.personal_info = False;
+
 
     @property
     def mandatory(self):
@@ -97,51 +109,73 @@ class Field:
     Emit new types required by this field
     """
     def emit_types(self, hfile, cfile):
-        if TypeFactory.is_type_emitted(self.fullname) is False:
+        if not TypeFactory.is_type_emitted(self.fullname):
             TypeFactory.set_type_emitted(self.fullname)
-            self.variable.emit_types(hfile, self.since, False)
-            self.variable.emit_helper_methods(hfile, cfile)
+            self.variable.emit_types(hfile, cfile, self.since, False)
+            if self.variable.needs_compat_gir and self.service != 'CTL':
+                self.variable.emit_types_gir(hfile, cfile, self.since)
 
 
     """
-    Emit the method responsible for getting this TLV from the input/output
-    container
+    Common getter logic
     """
-    def emit_getter(self, hfile, cfile):
-        input_variable_name = 'value_' + utils.build_underscore_name(self.name)
-        variable_getter_dec = self.variable.build_getter_declaration('    ', input_variable_name)
-        variable_getter_doc = self.variable.build_getter_documentation(' * ', input_variable_name)
-        variable_getter_imp = self.variable.build_getter_implementation('    ', 'self->' + self.variable_name, input_variable_name, True)
+    def emit_getter_common(self, hfile, cfile, dec, doc, imp, since, suffix_gir):
         translations = { 'name'                : self.name,
                          'variable_name'       : self.variable_name,
-                         'variable_getter_dec' : variable_getter_dec,
-                         'variable_getter_doc' : variable_getter_doc,
-                         'variable_getter_imp' : variable_getter_imp,
+                         'variable_getter_dec' : dec,
+                         'variable_getter_doc' : doc,
+                         'variable_getter_imp' : imp,
                          'underscore'          : utils.build_underscore_name(self.name),
+                         'suffix_gir'          : suffix_gir,
                          'prefix_camelcase'    : utils.build_camelcase_name(self.prefix),
                          'prefix_underscore'   : utils.build_underscore_name(self.prefix),
-                         'since'               : self.since,
+                         'since'               : since,
                          'static'              : 'static ' if self.static else '' }
 
         # Emit the getter header
         template = '\n'
-        if self.static == False:
+        if not self.static and self.service != 'CTL':
             template += (
                 '\n'
-                '/**\n'
-                ' * ${prefix_underscore}_get_${underscore}:\n'
+                '/**\n')
+
+            if self.variable.needs_compat_gir:
+                # Rename the GIR specific method to the original method name, and
+                # skip the original method completely
+                if suffix_gir == '':
+                    template += (
+                        ' * ${prefix_underscore}_get_${underscore}: (skip)\n')
+                else:
+                    template += (
+                        ' * ${prefix_underscore}_get_${underscore}${suffix_gir}: (rename-to ${prefix_underscore}_get_${underscore})\n')
+            else:
+                if suffix_gir != '':
+                    raise RuntimeError('GIR suffix given and variable does not need compat GIR support')
+                template += (
+                    ' * ${prefix_underscore}_get_${underscore}:\n')
+
+            template += (
                 ' * @self: a #${prefix_camelcase}.\n'
                 '${variable_getter_doc}'
                 ' * @error: Return location for error or %NULL.\n'
                 ' *\n'
-                ' * Get the \'${name}\' field from @self.\n'
+                ' * Get the \'${name}\' field from @self.\n')
+
+            if self.variable.needs_compat_gir:
+                if suffix_gir != '':
+                    template += (
+                        ' *\n'
+                        ' * Version of ${prefix_underscore}_get_${underscore}() using arrays of pointers to\n'
+                        ' * structs instead of arrays of structs, for easier binding in other languages.\n')
+
+            template += (
                 ' *\n'
-                ' * Returns: %TRUE if the field is found, %FALSE otherwise.\n'
+                ' * Returns: (skip): %TRUE if the field is found, %FALSE otherwise.\n'
                 ' *\n'
                 ' * Since: ${since}\n'
                 ' */\n')
         template += (
-            '${static}gboolean ${prefix_underscore}_get_${underscore} (\n'
+            '${static}gboolean ${prefix_underscore}_get_${underscore}${suffix_gir} (\n'
             '    ${prefix_camelcase} *self,\n'
             '${variable_getter_dec}'
             '    GError **error);\n')
@@ -151,7 +185,7 @@ class Field:
         template = (
             '\n'
             '${static}gboolean\n'
-            '${prefix_underscore}_get_${underscore} (\n'
+            '${prefix_underscore}_get_${underscore}${suffix_gir} (\n'
             '    ${prefix_camelcase} *self,\n'
             '${variable_getter_dec}'
             '    GError **error)\n'
@@ -174,44 +208,91 @@ class Field:
 
 
     """
-    Emit the method responsible for setting this TLV in the input/output
+    Emit the method responsible for declaring variable(s) for this TLV in the
+    input/output container
+    """
+    def emit_variable_declaration(self, cfile):
+
+        template = self.variable.build_variable_declaration('    ', self.variable_name)
+        cfile.write(template)
+
+        if self.variable.needs_compat_gir and self.service != 'CTL':
+            template = self.variable.build_variable_declaration_gir('    ', self.variable_name)
+            cfile.write(template)
+
+
+    """
+    Emit the method responsible for getting this TLV from the input/output
     container
     """
-    def emit_setter(self, hfile, cfile):
+    def emit_getter(self, hfile, cfile):
         input_variable_name = 'value_' + utils.build_underscore_name(self.name)
-        variable_setter_dec = self.variable.build_setter_declaration('    ', input_variable_name)
-        variable_setter_doc = self.variable.build_setter_documentation(' * ', input_variable_name)
-        variable_setter_imp = self.variable.build_setter_implementation('    ', input_variable_name, 'self->' + self.variable_name)
+        dec = self.variable.build_getter_declaration('    ', input_variable_name)
+        doc = self.variable.build_getter_documentation(' * ', input_variable_name)
+        imp = self.variable.build_getter_implementation('    ', 'self->' + self.variable_name, input_variable_name)
+        since = self.since
+        self.emit_getter_common(hfile, cfile, dec, doc, imp, since, '')
+
+        if self.variable.needs_compat_gir and self.service != 'CTL':
+            input_variable_name = 'value_' + utils.build_underscore_name(self.name)
+            dec = self.variable.build_getter_declaration_gir('    ', input_variable_name)
+            doc = self.variable.build_getter_documentation_gir(' * ', input_variable_name)
+            imp = self.variable.build_getter_implementation_gir('    ', 'self->' + self.variable_name, input_variable_name)
+            since = self.since if utils.version_compare('1.32',self.since) > 0 else '1.32'
+            self.emit_getter_common(hfile, cfile, dec, doc, imp, since, '_gir')
+
+
+    """
+    Common setter logic
+    """
+    def emit_setter_common(self, hfile, cfile, dec, doc, imp, since, suffix_gir):
         translations = { 'name'                : self.name,
                          'variable_name'       : self.variable_name,
-                         'variable_setter_dec' : variable_setter_dec,
-                         'variable_setter_doc' : variable_setter_doc,
-                         'variable_setter_imp' : variable_setter_imp,
+                         'variable_setter_dec' : dec,
+                         'variable_setter_doc' : doc,
+                         'variable_setter_imp' : imp,
                          'underscore'          : utils.build_underscore_name(self.name),
                          'prefix_camelcase'    : utils.build_camelcase_name(self.prefix),
                          'prefix_underscore'   : utils.build_underscore_name(self.prefix),
-                         'since'               : self.since,
+                         'since'               : since,
+                         'suffix_gir'          : suffix_gir,
                          'static'              : 'static ' if self.static else '' }
 
         # Emit the setter header
         template = '\n'
-        if self.static == False:
+        if not self.static and self.service != 'CTL':
             template += (
                 '\n'
-                '/**\n'
-                ' * ${prefix_underscore}_set_${underscore}:\n'
+                '/**\n')
+
+            if self.variable.needs_compat_gir:
+                # Rename the GIR specific method to the original method name, and
+                # skip the original method completely
+                if suffix_gir == '':
+                    template += (
+                        ' * ${prefix_underscore}_set_${underscore}: (skip)\n')
+                else:
+                    template += (
+                        ' * ${prefix_underscore}_set_${underscore}${suffix_gir}: (rename-to ${prefix_underscore}_set_${underscore})\n')
+            else:
+                if suffix_gir != '':
+                    raise RuntimeError('GIR suffix given and variable does not need compat GIR support')
+                template += (
+                    ' * ${prefix_underscore}_set_${underscore}:\n')
+
+            template += (
                 ' * @self: a #${prefix_camelcase}.\n'
                 '${variable_setter_doc}'
                 ' * @error: Return location for error or %NULL.\n'
                 ' *\n'
                 ' * Set the \'${name}\' field in the message.\n'
                 ' *\n'
-                ' * Returns: %TRUE if @value was successfully set, %FALSE otherwise.\n'
+                ' * Returns: (skip): %TRUE if @value was successfully set, %FALSE otherwise.\n'
                 ' *\n'
                 ' * Since: ${since}\n'
                 ' */\n')
         template += (
-            '${static}gboolean ${prefix_underscore}_set_${underscore} (\n'
+            '${static}gboolean ${prefix_underscore}_set_${underscore}${suffix_gir} (\n'
             '    ${prefix_camelcase} *self,\n'
             '${variable_setter_dec}'
             '    GError **error);\n')
@@ -221,7 +302,7 @@ class Field:
         template = (
             '\n'
             '${static}gboolean\n'
-            '${prefix_underscore}_set_${underscore} (\n'
+            '${prefix_underscore}_set_${underscore}${suffix_gir} (\n'
             '    ${prefix_camelcase} *self,\n'
             '${variable_setter_dec}'
             '    GError **error)\n'
@@ -235,6 +316,26 @@ class Field:
             '}\n')
         cfile.write(string.Template(template).substitute(translations))
 
+    """
+    Emit the method responsible for setting this TLV in the input/output
+    container
+    """
+    def emit_setter(self, hfile, cfile):
+        input_variable_name = 'value_' + utils.build_underscore_name(self.name)
+        dec = self.variable.build_setter_declaration('    ', input_variable_name)
+        doc = self.variable.build_setter_documentation(' * ', input_variable_name)
+        imp = self.variable.build_setter_implementation('    ', input_variable_name, 'self->' + self.variable_name)
+        since = self.since
+        self.emit_setter_common(hfile, cfile, dec, doc, imp, since, '')
+
+        if self.variable.needs_compat_gir and self.service != 'CTL':
+            input_variable_name = 'value_' + utils.build_underscore_name(self.name)
+            dec = self.variable.build_setter_declaration_gir('    ', input_variable_name)
+            doc = self.variable.build_setter_documentation_gir(' * ', input_variable_name)
+            imp = self.variable.build_setter_implementation_gir('    ', input_variable_name, 'self->' + self.variable_name)
+            since = self.since if utils.version_compare('1.32',self.since) > 0 else '1.32'
+            self.emit_setter_common(hfile, cfile, dec, doc, imp, since, '_gir')
+
 
     """
     Emit the code responsible for adding the TLV to the QMI message
@@ -327,7 +428,7 @@ class Field:
         template = (
             '\n'
             '${lp}/* The remaining size of the buffer needs to be 0 if we successfully read the TLV */\n'
-            '${lp}if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {\n'
+            '${lp}if ((offset = qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {\n'
             '${lp}    g_warning ("Left \'%" G_GSIZE_FORMAT "\' bytes unread when getting the \'${name}\' TLV", offset);\n'
             '${lp}}\n'
             '\n'
@@ -378,11 +479,11 @@ class Field:
         f.write(string.Template(template).substitute(translations))
 
         # Now, read the contents of the buffer into the printable representation
-        self.variable.emit_get_printable(f, '    ')
+        self.variable.emit_get_printable(f, '    ', self.personal_info)
 
         template = (
             '\n'
-            '    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)\n'
+            '    if ((offset = qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)\n'
             '        g_string_append_printf (printable, "Additional unexpected \'%" G_GSIZE_FORMAT "\' bytes", offset);\n'
             '\n'
             'out:\n'
@@ -403,14 +504,20 @@ class Field:
                          'prefix_camelcase'  : utils.build_camelcase_name(self.prefix),
                          'prefix_underscore' : utils.build_underscore_name(self.prefix) }
 
-        if TypeFactory.is_section_emitted(self.fullname) is False:
+        if not TypeFactory.is_section_emitted(self.fullname):
             TypeFactory.set_section_emitted(self.fullname)
             self.variable.add_sections(sections)
 
         # Public methods
         template = (
             '${prefix_underscore}_get_${underscore}\n')
+        if self.variable.needs_compat_gir and self.service != 'CTL':
+            template += (
+                '${prefix_underscore}_get_${underscore}_gir\n')
         if self.container_type == 'Input':
             template += (
                 '${prefix_underscore}_set_${underscore}\n')
+            if self.variable.needs_compat_gir and self.service != 'CTL':
+                template += (
+                    '${prefix_underscore}_set_${underscore}_gir\n')
         sections['public-methods'] += string.Template(template).substitute(translations)
