@@ -16,6 +16,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright (C) 2012-2017 Aleksander Morgado <aleksander@aleksander.es>
+ * Copyright (c) 2022 Qualcomm Innovation Center, Inc.
  */
 
 #include "config.h"
@@ -68,6 +69,7 @@ static gboolean device_open_auto_flag;
 static gchar *client_cid_str;
 static gboolean client_no_release_cid_flag;
 static gboolean verbose_flag;
+static gboolean verbose_full_flag;
 static gboolean silent_flag;
 static gboolean version_flag;
 
@@ -129,6 +131,10 @@ static GOptionEntry main_entries[] = {
       "Run action with verbose logs, including the debug ones",
       NULL
     },
+    { "verbose-full", 0, 0, G_OPTION_ARG_NONE, &verbose_full_flag,
+      "Run action with verbose logs, including the debug ones and personal info",
+      NULL
+    },
     { "silent", 0, 0, G_OPTION_ARG_NONE, &silent_flag,
       "Run action with no logs; not even the error/warning ones",
       NULL
@@ -210,7 +216,7 @@ log_handler (const gchar *log_domain,
         g_assert_not_reached ();
     }
 
-    if (!verbose_flag && !err)
+    if (!verbose_flag && !verbose_full_flag && !err)
         return;
 
     g_fprintf (err ? stderr : stdout,
@@ -225,7 +231,7 @@ static void
 print_version_and_exit (void)
 {
     g_print (PROGRAM_NAME " " PROGRAM_VERSION "\n"
-             "Copyright (C) 2012-2021 Aleksander Morgado\n"
+             "Copyright (C) 2012-2022 Aleksander Morgado\n"
              "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl-2.0.html>\n"
              "This is free software: you are free to change and redistribute it.\n"
              "There is NO WARRANTY, to the extent permitted by law.\n"
@@ -455,6 +461,20 @@ allocate_client_ready (QmiDevice *dev,
         return;
 #else
         break;
+#endif
+    case QMI_SERVICE_DPM:
+#if defined HAVE_QMI_SERVICE_DPM
+        qmicli_dpm_run (dev, QMI_CLIENT_DPM (client), cancellable);
+        return;
+#else
+        break;
+#endif
+    case QMI_SERVICE_FOX:
+#if defined HAVE_QMI_SERVICE_FOX
+        qmicli_fox_run (dev, QMI_CLIENT_FOX (client), cancellable);
+        return;
+#else
+        break;
 #endif
     case QMI_SERVICE_UNKNOWN:
     case QMI_SERVICE_CTL:
@@ -899,6 +919,20 @@ parse_actions (void)
     }
 #endif
 
+#if defined HAVE_QMI_SERVICE_DPM
+    if (qmicli_dpm_options_enabled ()) {
+        service = QMI_SERVICE_DPM;
+        actions_enabled++;
+    }
+#endif
+
+#if defined HAVE_QMI_SERVICE_FOX
+    if (qmicli_fox_options_enabled ()) {
+        service = QMI_SERVICE_FOX;
+        actions_enabled++;
+    }
+#endif
+
     /* Cannot mix actions from different services */
     if (actions_enabled > 1) {
         g_printerr ("error: cannot execute multiple actions of different services\n");
@@ -968,6 +1002,12 @@ int main (int argc, char **argv)
 #endif
 #if defined HAVE_QMI_SERVICE_DSD
     g_option_context_add_group (context, qmicli_dsd_get_option_group ());
+#endif
+#if defined HAVE_QMI_SERVICE_DPM
+    g_option_context_add_group (context, qmicli_dpm_get_option_group ());
+#endif
+#if defined HAVE_QMI_SERVICE_FOX
+    g_option_context_add_group (context, qmicli_fox_get_option_group ());
 #endif
     g_option_context_add_group (context, qmicli_link_management_get_option_group ());
     g_option_context_add_group (context, qmicli_qmiwwan_get_option_group ());
@@ -984,14 +1024,32 @@ int main (int argc, char **argv)
 
     g_log_set_handler (NULL, G_LOG_LEVEL_MASK, log_handler, NULL);
     g_log_set_handler ("Qmi", G_LOG_LEVEL_MASK, log_handler, NULL);
-    if (verbose_flag)
+
+    if (verbose_flag && verbose_full_flag) {
+        g_printerr ("error: cannot specify --verbose and --verbose-full at the same time\n");
+        exit (EXIT_FAILURE);
+    } else if (verbose_flag) {
         qmi_utils_set_traces_enabled (TRUE);
+        qmi_utils_set_show_personal_info (FALSE);
+    } else if (verbose_full_flag) {
+        qmi_utils_set_traces_enabled (TRUE);
+        qmi_utils_set_show_personal_info (TRUE);
+    }
 
 #if QMI_MBIM_QMUX_SUPPORTED
     /* libmbim logging */
     g_log_set_handler ("Mbim", G_LOG_LEVEL_MASK, log_handler, NULL);
-    if (verbose_flag)
+    if (verbose_flag) {
         mbim_utils_set_traces_enabled (TRUE);
+#if MBIM_CHECK_VERSION(1,27,6)
+        mbim_utils_set_show_personal_info (FALSE);
+#endif
+    } else if (verbose_full_flag) {
+        mbim_utils_set_traces_enabled (TRUE);
+#if MBIM_CHECK_VERSION(1,27,6)
+        mbim_utils_set_show_personal_info (TRUE);
+#endif
+    }
 #endif
 
 #if QMI_QRTR_SUPPORTED
